@@ -100,32 +100,37 @@ poppler_attachment_finalize (GObject *obj)
 /* Public functions */
 
 PopplerAttachment *
-_poppler_attachment_new (EmbFile *emb_file)
+_poppler_attachment_new (FileSpec *emb_file)
 {
   PopplerAttachment *attachment;
   PopplerAttachmentPrivate *priv;
+  EmbFile *embFile;
 
   g_assert (emb_file != NULL);
 
   attachment = (PopplerAttachment *) g_object_new (POPPLER_TYPE_ATTACHMENT, NULL);
   priv = POPPLER_ATTACHMENT_GET_PRIVATE (attachment);
 
-  if (emb_file->name ())
-    attachment->name = _poppler_goo_string_to_utf8 (emb_file->name ());
-  if (emb_file->description ())
-    attachment->description = _poppler_goo_string_to_utf8 (emb_file->description ());
+  if (emb_file->getFileName ())
+    attachment->name = _poppler_goo_string_to_utf8 (emb_file->getFileName ());
+  if (emb_file->getDescription ())
+    attachment->description = _poppler_goo_string_to_utf8 (emb_file->getDescription ());
 
-  attachment->size = emb_file->size ();
-  
-  _poppler_convert_pdf_date_to_gtime (emb_file->createDate (), (time_t *)&attachment->ctime);
-  _poppler_convert_pdf_date_to_gtime (emb_file->modDate (), (time_t *)&attachment->mtime);
+  embFile = emb_file->getEmbeddedFile();
+  attachment->size = embFile->size ();
 
-  if (emb_file->checksum ()->getLength () > 0)
-	  attachment->checksum = g_string_new_len (emb_file->checksum ()->getCString (),
-						   emb_file->checksum ()->getLength ());
-  
+  if (embFile->createDate ())
+    _poppler_convert_pdf_date_to_gtime (embFile->createDate (), (time_t *)&attachment->ctime);
+  if (embFile->modDate ())
+    _poppler_convert_pdf_date_to_gtime (embFile->modDate (), (time_t *)&attachment->mtime);
+
+  if (embFile->checksum () && embFile->checksum ()->getLength () > 0)
+    attachment->checksum = g_string_new_len (embFile->checksum ()->getCString (),
+                                             embFile->checksum ()->getLength ());
   priv->obj_stream = new Object();
-  emb_file->streamObject().copy(priv->obj_stream);
+  priv->obj_stream->initStream(embFile->stream());
+  // Copy the stream
+  embFile->stream()->incRef();
 
   return attachment;
 }
@@ -213,7 +218,7 @@ poppler_attachment_save (PopplerAttachment  *attachment,
 /**
  * poppler_attachment_save_to_callback:
  * @attachment: A #PopplerAttachment.
- * @save_func: a function that is called to save each block of data that the save routine generates.
+ * @save_func: (scope call): a function that is called to save each block of data that the save routine generates.
  * @user_data: user data to pass to the save function.
  * @error: (allow-none): return location for error, or %NULL.
  * 
